
package ONVIF::Media::Elements::GetVideoSourceModesResponse;
use strict;
use warnings;

{ # BLOCK to scope variables

sub get_xmlns { 'http://www.onvif.org/ver10/media/wsdl' }

__PACKAGE__->__set_name('GetVideoSourceModesResponse');
__PACKAGE__->__set_nillable();
__PACKAGE__->__set_minOccurs();
__PACKAGE__->__set_maxOccurs();
__PACKAGE__->__set_ref();

use base qw(
    SOAP::WSDL::XSD::Typelib::Element
    SOAP::WSDL::XSD::Typelib::ComplexType
);

our $XML_ATTRIBUTE_CLASS;
undef $XML_ATTRIBUTE_CLASS;

sub __get_attr_class {
    return $XML_ATTRIBUTE_CLASS;
}

use Class::Std::Fast::Storable constructor => 'none';
use base qw(SOAP::WSDL::XSD::Typelib::ComplexType);

Class::Std::initialize();

{ # BLOCK to scope variables

my %VideoSourceModes_of :ATTR(:get<VideoSourceModes>);

__PACKAGE__->_factory(
    [ qw(        VideoSourceModes

    ) ],
    {
        'VideoSourceModes' => \%VideoSourceModes_of,
    },
    {
        'VideoSourceModes' => 'ONVIF::Media::Types::VideoSourceMode',
    },
    {

        'VideoSourceModes' => 'VideoSourceModes',
    }
);

} # end BLOCK







} # end of BLOCK



1;


=pod

=head1 NAME

ONVIF::Media::Elements::GetVideoSourceModesResponse

=head1 DESCRIPTION

Perl data type class for the XML Schema defined element
GetVideoSourceModesResponse from the namespace http://www.onvif.org/ver10/media/wsdl.







=head1 PROPERTIES

The following properties may be accessed using get_PROPERTY / set_PROPERTY
methods:

=over

=item * VideoSourceModes

 $element->set_VideoSourceModes($data);
 $element->get_VideoSourceModes();





=back


=head1 METHODS

=head2 new

 my $element = ONVIF::Media::Elements::GetVideoSourceModesResponse->new($data);

Constructor. The following data structure may be passed to new():

 {
   VideoSourceModes =>  { # ONVIF::Media::Types::VideoSourceMode
     MaxFramerate =>  $some_value, # float
     MaxResolution =>  { # ONVIF::Media::Types::VideoResolution
       Width =>  $some_value, # int
       Height =>  $some_value, # int
     },
     Encodings => $some_value, # EncodingTypes
     Reboot =>  $some_value, # boolean
     Description => $some_value, # Description
     Extension =>  { # ONVIF::Media::Types::VideoSourceModeExtension
     },
   },
 },

=head1 AUTHOR

Generated by SOAP::WSDL

=cut

