@import url(https://fonts.googleapis.com/css2?family=Poppins:wght@100;200;300;400;500;600;700;800;900&display=swap);
/*@import 'css-pro-layout/dist/scss/css-pro-layout.scss';*/
.layout-main {
  min-height: 100%;
  display: flex;
  flex-direction: column;
  position: relative;
  flex-grow: 1;
}
.layout-main.has-sidebar {
  flex-direction: row;
}
.layout-main .header {
  transition: width, 0.3s;
  height: 64px;
  min-height: 64px;
  position: relative;
}
.layout-main .sidebar-main {
  width: 250px;
  min-width: 250px;
  transition: width, left, right, 0.3s;
}
.layout-main .sidebar-main.collapsed {
  width: 80px;
  min-width: 80px;
}
@media (max-width: 480px) {
  .layout-main .sidebar-main.break-point-xs {
    position: fixed;
    left: -250px;
    height: 100%;
    top: 0;
    z-index: 100;
  }
  .layout-main .sidebar-main.break-point-xs.collapsed {
    left: -80px;
  }
  .layout-main .sidebar-main.break-point-xs.toggled {
    left: 0;
  }
  .layout-main .sidebar-main.break-point-xs.toggled ~ .overlay {
    display: block;
  }
  .layout-main .sidebar-main.break-point-xs ~ .layout-main .header {
    width: 100% !important;
    transition: none;
  }
}
@media (max-width: 576px) {
  .layout-main .sidebar-main.break-point-sm {
    position: fixed;
    left: -250px;
    height: 100%;
    top: 0;
    z-index: 100;
  }
  .layout-main .sidebar-main.break-point-sm.collapsed {
    left: -80px;
  }
  .layout-main .sidebar-main.break-point-sm.toggled {
    left: 0;
  }
  .layout-main .sidebar-main.break-point-sm.toggled ~ .overlay {
    display: block;
  }
  .layout-main .sidebar-main.break-point-sm ~ .layout-main .header {
    width: 100% !important;
    transition: none;
  }
}
@media (max-width: 768px) {
  .layout-main .sidebar-main.break-point-md {
    position: fixed;
    left: -250px;
    height: 100%;
    top: 0;
    z-index: 100;
  }
  .layout-main .sidebar-main.break-point-md.collapsed {
    left: -80px;
  }
  .layout-main .sidebar-main.break-point-md.toggled {
    left: 0;
  }
  .layout-main .sidebar-main.break-point-md.toggled ~ .overlay {
    display: block;
  }
  .layout-main .sidebar-main.break-point-md ~ .layout-main .header {
    width: 100% !important;
    transition: none;
  }
}
@media (max-width: 992px) {
  .layout-main .sidebar-main.break-point-lg {
    position: fixed;
    left: -250px;
    height: 100%;
    top: 0;
    z-index: 100;
  }
  .layout-main .sidebar-main.break-point-lg.collapsed {
    left: -80px;
  }
  .layout-main .sidebar-main.break-point-lg.toggled {
    left: 0;
  }
  .layout-main .sidebar-main.break-point-lg.toggled ~ .overlay {
    display: block;
  }
  .layout-main .sidebar-main.break-point-lg ~ .layout-main .header {
    width: 100% !important;
    transition: none;
  }
}
@media (max-width: 1200px) {
  .layout-main .sidebar-main.break-point-xl {
    position: fixed;
    left: -250px;
    height: 100%;
    top: 0;
    z-index: 100;
  }
  .layout-main .sidebar-main.break-point-xl.collapsed {
    left: -80px;
  }
  .layout-main .sidebar-main.break-point-xl.toggled {
    left: 0;
  }
  .layout-main .sidebar-main.break-point-xl.toggled ~ .overlay {
    display: block;
  }
  .layout-main .sidebar-main.break-point-xl ~ .layout-main .header {
    width: 100% !important;
    transition: none;
  }
}
@media (max-width: 1600px) {
  .layout-main .sidebar-main.break-point-xxl {
    position: fixed;
    left: -250px;
    height: 100%;
    top: 0;
    z-index: 100;
  }
  .layout-main .sidebar-main.break-point-xxl.collapsed {
    left: -80px;
  }
  .layout-main .sidebar-main.break-point-xxl.toggled {
    left: 0;
  }
  .layout-main .sidebar-main.break-point-xxl.toggled ~ .overlay {
    display: block;
  }
  .layout-main .sidebar-main.break-point-xxl ~ .layout-main .header {
    width: 100% !important;
    transition: none;
  }
}
.layout-main .footer {
  height: 64px;
  min-height: 64px;
}
.layout-main .content {
  flex-grow: 1;
}
.layout-main .overlay {
  position: fixed;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  background-color: rgba(0, 0, 0, 0.3);
  z-index: 99;
  display: none;
}
.layout-main .sidebar-toggler {
  display: none;
}
@media (max-width: 480px) {
  .layout-main .sidebar-toggler.break-point-xs {
    display: initial;
  }
}
@media (max-width: 576px) {
  .layout-main .sidebar-toggler.break-point-sm {
    display: initial;
  }
}
@media (max-width: 768px) {
  .layout-main .sidebar-toggler.break-point-md {
    display: initial;
  }
}
@media (max-width: 992px) {
  .layout-main .sidebar-toggler.break-point-lg {
    display: initial;
  }
}
@media (max-width: 1200px) {
  .layout-main .sidebar-toggler.break-point-xl {
    display: initial;
  }
}
@media (max-width: 1600px) {
  .layout-main .sidebar-toggler.break-point-xxl {
    display: initial;
  }
}
.layout-main.fixed-sidebar {
  height: 100%;
}
.layout-main.fixed-sidebar .sidebar {
  height: 100%;
  overflow: auto;
}
.layout-main.fixed-sidebar .sidebar ~ .layout-main {
  height: 100%;
  overflow: auto;
}
.layout-main.fixed-header .header {
  position: fixed;
  width: 100%;
  z-index: 2;
}
.layout-main.fixed-header .header ~ .layout-main,
.layout-main.fixed-header .header ~ .content {
  margin-top: 64px;
}
.layout-main.fixed-header.fixed-sidebar .header {
  width: calc(100% - 250px);
}
.layout-main.fixed-header.fixed-sidebar .sidebar.collapsed ~ .layout-main .header {
  width: calc(100% - 80px);
}
.layout-main.rtl {
  direction: rtl;
}
@media (max-width: 480px) {
  .layout-main.rtl .sidebar.break-point-xs {
    left: auto;
    right: -250px;
  }
  .layout-main.rtl .sidebar.break-point-xs.collapsed {
    left: auto;
    right: -80px;
  }
  .layout-main.rtl .sidebar.break-point-xs.toggled {
    left: auto;
    right: 0;
  }
}
@media (max-width: 576px) {
  .layout-main.rtl .sidebar.break-point-sm {
    left: auto;
    right: -250px;
  }
  .layout-main.rtl .sidebar.break-point-sm.collapsed {
    left: auto;
    right: -80px;
  }
  .layout-main.rtl .sidebar.break-point-sm.toggled {
    left: auto;
    right: 0;
  }
}
@media (max-width: 768px) {
  .layout-main.rtl .sidebar.break-point-md {
    left: auto;
    right: -250px;
  }
  .layout-main.rtl .sidebar.break-point-md.collapsed {
    left: auto;
    right: -80px;
  }
  .layout-main.rtl .sidebar.break-point-md.toggled {
    left: auto;
    right: 0;
  }
}
@media (max-width: 992px) {
  .layout-main.rtl .sidebar.break-point-lg {
    left: auto;
    right: -250px;
  }
  .layout-main.rtl .sidebar.break-point-lg.collapsed {
    left: auto;
    right: -80px;
  }
  .layout-main.rtl .sidebar.break-point-lg.toggled {
    left: auto;
    right: 0;
  }
}
@media (max-width: 1200px) {
  .layout-main.rtl .sidebar.break-point-xl {
    left: auto;
    right: -250px;
  }
  .layout-main.rtl .sidebar.break-point-xl.collapsed {
    left: auto;
    right: -80px;
  }
  .layout-main.rtl .sidebar.break-point-xl.toggled {
    left: auto;
    right: 0;
  }
}
@media (max-width: 1600px) {
  .layout-main.rtl .sidebar.break-point-xxl {
    left: auto;
    right: -250px;
  }
  .layout-main.rtl .sidebar.break-point-xxl.collapsed {
    left: auto;
    right: -80px;
  }
  .layout-main.rtl .sidebar.break-point-xxl.toggled {
    left: auto;
    right: 0;
  }
}

/*@import '../../public//assets//remixicon/remixicon.css';*/
/*
.layout {
  z-index: 1;
  .header {
    display: flex;
    align-items: center;
    padding: 20px;
  }
  .content {
    padding: 12px 50px;
    display: flex;
    flex-direction: column;
  }

  .footer {
    text-align: center;
    margin-top: auto;
    margin-bottom: 20px;
    padding: 20px;
  }
}
*/
.sidebar-main {
  color: #44b8eb;
  position: relative;
  height: 100%;
  max-height: 100%;
  overflow: hidden !important;
}
.sidebar-main .image-wrapper {
  overflow: hidden;
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  z-index: 1;
  display: none;
}
.sidebar-main .image-wrapper > img {
  width: 100%;
  height: 100%;
  object-fit: cover;
  object-position: center;
}
.sidebar-main.has-bg-image .sidebar-layout {
  background-color: rgba(12, 30, 53, 0.85);
}
.sidebar-main.has-bg-image .image-wrapper {
  display: block;
  opacity: 0.8;
}
.sidebar-main .sidebar-layout {
  height: 100%;
  max-height: 100%;
  min-height: 100%;
  overflow-x: hidden;
  overflow-y: auto;
  display: flex;
  flex-direction: column;
  position: relative;
  background-color: #0c1e35;
  z-index: 2;
  opacity: 0.85;
}
.sidebar-main .sidebar-layout::-webkit-scrollbar-thumb {
  border-radius: 4px;
}
.sidebar-main .sidebar-layout:hover::-webkit-scrollbar-thumb {
  background-color: rgb(26.1230769231, 65.3076923077, 115.3769230769);
}
.sidebar-main .sidebar-layout::-webkit-scrollbar {
  width: 6px;
  background-color: #0c1e35;
}
.sidebar-main .sidebar-layout .sidebar-header {
  height: 40px;
  min-height: 40px;
  display: flex;
  align-items: center;
  padding: 8px 22px;
}
.sidebar-main .sidebar-layout .sidebar-header > span {
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
}
.sidebar-main .sidebar-layout .sidebar-content {
  flex-grow: 1;
  padding: 10px 0;
}
.sidebar-main .sidebar-layout .sidebar-footer {
  height: 230px;
  min-height: 230px;
  display: flex;
  align-items: center;
  padding: 0 20px;
}
.sidebar-main .sidebar-layout .sidebar-footer > span {
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
}

@keyframes swing {
  0%, 30%, 50%, 70%, 100% {
    transform: rotate(0deg);
  }
  10% {
    transform: rotate(10deg);
  }
  40% {
    transform: rotate(-10deg);
  }
  60% {
    transform: rotate(5deg);
  }
  80% {
    transform: rotate(-5deg);
  }
}
.layout-main .sidebar-main .sidebar-main-menu ul {
  list-style-type: none;
  padding: 0;
  margin: 0;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-header {
  font-weight: 600;
  padding: 10px 25px;
  font-size: 0.8em;
  letter-spacing: 2px;
  transition: opacity 0.3s;
  opacity: 0.5;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item a {
  display: flex;
  align-items: center;
  height: 38px;
  padding: 0 20px;
  color: #44b8eb;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item a .menu-icon {
  font-size: 1.2rem;
  width: 35px;
  min-width: 35px;
  height: 35px;
  line-height: 35px;
  text-align: center;
  display: inline-block;
  margin-right: 10px;
  border-radius: 2px;
  transition: color 0.3s;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item a .menu-title {
  font-size: 0.9em;
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
  flex-grow: 1;
  transition: color 0.3s;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item a .menu-prefix,
.layout-main .sidebar-main .sidebar-main-menu .menu-item a .menu-suffix {
  display: inline-block;
  padding: 5px;
  opacity: 1;
  transition: opacity 0.3s;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item a:hover .menu-title {
  color: #dee2ec;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item a:hover .menu-icon {
  color: #dee2ec;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item a:hover .menu-icon i {
  animation: swing ease-in-out 0.5s 1 alternate;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item a:hover::after {
  border-color: #dee2ec !important;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item.sub-menu {
  position: relative;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item.sub-menu > a::after {
  content: "";
  transition: transform 0.3s;
  border-right: 2px solid currentcolor;
  border-bottom: 2px solid currentcolor;
  width: 5px;
  height: 5px;
  transform: rotate(-45deg);
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item.sub-menu > .sub-menu-list {
  padding-left: 20px;
  display: none;
  overflow: hidden;
  z-index: 999;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item.sub-menu.open > a {
  color: #dee2ec;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item.sub-menu.open > a::after {
  transform: rotate(45deg);
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item.active > a .menu-title {
  color: #dee2ec;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item.active > a::after {
  border-color: #dee2ec;
}
.layout-main .sidebar-main .sidebar-main-menu .menu-item.active > a .menu-icon {
  color: #dee2ec;
}
.layout-main .sidebar-main .sidebar-main-menu > ul > .sub-menu > .sub-menu-list {
  background-color: #0b1a2c;
}
.layout-main .sidebar-main .sidebar-main-menu.icon-shape-circle .menu-item a .menu-icon, .layout-main .sidebar-main .sidebar-main-menu.icon-shape-rounded .menu-item a .menu-icon, .layout-main .sidebar-main .sidebar-main-menu.icon-shape-square .menu-item a .menu-icon {
  background-color: #0b1a2c;
}
.layout-main .sidebar-main .sidebar-main-menu.icon-shape-circle .menu-item a .menu-icon {
  border-radius: 50%;
}
.layout-main .sidebar-main .sidebar-main-menu.icon-shape-rounded .menu-item a .menu-icon {
  border-radius: 4px;
}
.layout-main .sidebar-main .sidebar-main-menu.icon-shape-square .menu-item a .menu-icon {
  border-radius: 0;
}
.layout-main .sidebar-main:not(.collapsed) .sidebar-main-menu > ul > .menu-item.sub-menu > .sub-menu-list {
  visibility: visible !important;
  position: static !important;
  transform: translate(0, 0) !important;
}
.layout-main .sidebar-main.collapsed .sidebar-main-menu > ul > .menu-header {
  opacity: 0;
}
.layout-main .sidebar-main.collapsed .sidebar-main-menu > ul > .menu-item > a .menu-prefix,
.layout-main .sidebar-main.collapsed .sidebar-main-menu > ul > .menu-item > a .menu-suffix {
  opacity: 0;
}
.layout-main .sidebar-main.collapsed .sidebar-main-menu > ul > .menu-item.sub-menu > a::after {
  content: "";
  width: 5px;
  height: 5px;
  background-color: currentcolor;
  border-radius: 50%;
  display: inline-block;
  position: absolute;
  right: 10px;
  top: 50%;
  border: none;
  transform: translateY(-50%);
}
.layout-main .sidebar-main.collapsed .sidebar-main-menu > ul > .menu-item.sub-menu > a:hover::after {
  background-color: #dee2ec;
}
.layout-main .sidebar-main.collapsed .sidebar-main-menu > ul > .menu-item.sub-menu > .sub-menu-list {
  transition: none !important;
  width: 200px;
  margin-left: 3px !important;
  border-radius: 4px;
  display: block !important;
}
.layout-main .sidebar-main.collapsed .sidebar-main-menu > ul > .menu-item.active > a::after {
  background-color: #dee2ec;
}
.layout-main .sidebar-main.has-bg-image .sidebar-main-menu.icon-shape-circle .menu-item a .menu-icon, .layout-main .sidebar-main.has-bg-image .sidebar-main-menu.icon-shape-rounded .menu-item a .menu-icon, .layout-main .sidebar-main.has-bg-image .sidebar-main-menu.icon-shape-square .menu-item a .menu-icon {
  background-color: rgba(11, 26, 44, 0.6);
}
.layout-main .sidebar-main.has-bg-image:not(.collapsed) .sidebar-main-menu > ul > .sub-menu > .sub-menu-list {
  background-color: rgba(11, 26, 44, 0.6);
}
.layout-main.rtl .sidebar-main .sidebar-main-menu .menu-item a .menu-icon {
  margin-left: 10px;
  margin-right: 0;
}
.layout-main.rtl .sidebar-main .sidebar-main-menu .menu-item.sub-menu > a::after {
  transform: rotate(135deg);
}
.layout-main.rtl .sidebar-main .sidebar-main-menu .menu-item.sub-menu > .sub-menu-list {
  padding-left: 0;
  padding-right: 20px;
}
.layout-main.rtl .sidebar-main .sidebar-main-menu .menu-item.sub-menu.open > a::after {
  transform: rotate(45deg);
}
.layout-main.rtl .sidebar-main.collapsed .sidebar-main-menu > ul > .menu-item.sub-menu a::after {
  right: auto;
  left: 10px;
}
.layout-main.rtl .sidebar-main.collapsed .sidebar-main-menu > ul > .menu-item.sub-menu > .sub-menu-list {
  margin-left: -3px !important;
}

/*
* {
  box-sizing: border-box;
}

body {
  margin: 0;
  height: 100vh;
  font-family: 'Poppins', sans-serif;
  color: #3f4750;
  font-size: 0.9rem;
}
*/
a {
  text-decoration: none;
}

@media (max-width: 992px) {
  #btn-collapse {
    display: none;
  }
}
.layout-main .sidebar-main .pro-sidebar-logo {
  color: #ffa801;
  display: flex;
  align-items: center;
}
.layout-main .sidebar-main .pro-sidebar-logo > div {
  width: 45px;
  min-width: 45px;
  height: 25px;
  min-height: 25px;
  display: flex;
  align-items: center;
  justify-content: center;
  border-radius: 8px;
  color: white;
  font-size: 22px;
  font-weight: 700;
  background-color: #ffa801;
  margin-right: 10px;
  margin-left: -10px;
}
.layout-main .sidebar-main .pro-sidebar-logo > h5 {
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
  font-size: 22px;
  line-height: 25px;
  transition: opacity 0.3s;
  opacity: 1;
  margin: 0 17px;
}
.layout-main .sidebar-main .footer-box {
  display: flex;
  flex-direction: column;
  align-items: center;
  justify-content: center;
  text-align: center;
  /* font-size: 0.8em; */
  padding: 20px 0;
  border-radius: 8px;
  width: 180px;
  min-width: 190px;
  margin: 0 auto;
  background-color: #162d4a;
}
.layout-main .sidebar-main .footer-box img.react-logo {
  width: 40px;
  height: 40px;
  margin-bottom: 10px;
}
.layout-main .sidebar-main .footer-box a {
  color: #fff;
  font-weight: 600;
  margin-bottom: 10px;
}
.layout-main .sidebar-main .sidebar-collapser {
  transition: left, right, 0.3s;
  position: fixed;
  left: calc(250px - 20px);
  top: 40px;
  width: 20px;
  height: 20px;
  border-radius: 50%;
  background-color: #00829f;
  display: flex;
  align-items: center;
  justify-content: center;
  font-size: 1.2em;
  transform: translateX(50%);
  z-index: 111;
  cursor: pointer;
  color: white;
  box-shadow: 1px 1px 4px #0c1e35;
}
.layout-main .sidebar-main.has-bg-image .footer-box {
  /* background-color: rgba(#162d4a, 0.7); */
  background-color: #34495e;
}
.layout-main .sidebar-main.collapsed .pro-sidebar-logo > h5 {
  opacity: 0;
}
.layout-main .sidebar-main.collapsed .footer-box {
  display: none;
}
.layout-main .sidebar-main.collapsed .sidebar-collapser {
  left: calc(80px - 20px);
}
.layout-main .sidebar-main.collapsed .sidebar-collapser i {
  transform: rotate(180deg);
}

.badge {
  display: inline-block;
  padding: 0.25em 0.4em;
  font-size: 75%;
  font-weight: 700;
  line-height: 1;
  text-align: center;
  white-space: nowrap;
  vertical-align: baseline;
  border-radius: 0.25rem;
  color: #fff;
  background-color: #6c757d;
}
.badge.primary {
  background-color: #ab2dff;
}
.badge.secondary {
  background-color: #079b0b;
}

.sidebar-toggler {
  position: fixed;
  right: 20px;
  top: 20px;
}

.social-links a {
  margin: 0 10px;
  color: #3f4750;
}

/*# sourceMappingURL=main.css.map*/