/*
 * ZoneMinder Base Stylesheet, $Date$, $Revision$
 * Copyright (C) 2001-2008 Philip Coombes
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */ 

/*
 * Primary look and feel styles
 */

:root {
  --scrollbarBG: #F1F1F1;
  --sliderBG: #C1C1C1;
  --alarmBG: #FFC0C0;
  --alarmText: inherit;
  --backgroundMediumDark: #485460;
  --colorLink: #3498db;
  --colorBackgroundButtons: #3498db;
  --colorBorder: #3498db;
}

html,
body {
  height: 100%;
  margin: 0;
}

body {
  font-family: "Open Sans", Verdana, Arial, Helvetica, sans-serif;
  font-size: 13px;
  font-weight: 300;
  text-align: center;
}

body.sticky {
  display: flex;
  flex-flow: column nowrap;
  flex-direction: column;
}

h1 {
    font-family: inherit;
    font-size: 120%;
    font-weight: bold;
    text-align: center;
}

h2 {
    font-family: inherit;
    font-size: 110%;
    font-weight: bold;
}

h3 {
    font-family: inherit;
    font-size: 100%;
    font-weight: bold;
}

h4 {
    font-family: inherit;
    font-size: 100%;
}

p {
    font-family: inherit;
    font-size: 100%;
    font-weight: normal;
}

/*
th {
    padding: 3px;
    text-transform: uppercase;
    font-size: 0.8em;
    font-weight: 600;
}
*/

table thead {
  z-index: 10;
}

.thead-highlight {
  background-color:#dfe4ea;
}

/* rounded table head top corners */
table th:first-child{
  border-radius:5px 0 0 0 ;
}
table th:last-child{
  border-radius:0 5px 0 0 ;
}

a {
  cursor: pointer;
}

a.optionhelp,
a:link {
    color: var(--colorLink);
    text-decoration: none;
}

a:visited {
    color: #44b8eb;
    text-decoration: none;
}

a:hover {
    text-decoration: none; 
    background-color:rgba(72, 84, 96,0.2);
    border-radius:4px;
}

a:not([href]):hover {
    color: #212529;
}

label {
    margin: 0 4px;
}

button {
  line-height: 18.5px;
}

a.btn,
button.btn {
  line-height: 1;
  font-size: 18px;
  margin-bottom: 3px;
}
#toolbar .btn-normal,
#toolbar .btn-danger {
  margin-right: 3px;
}
#rightButtons .btn-normal,
#leftButtons .btn-normal {
  margin-right: 0px;
}
input, textarea, select, button, .btn-primary {
    border: 1px #ccc solid;
    padding: 5px;
    border-radius: 1px;
    font-family: inherit;
    font-weight: 400;
    font-size: 100%;
    color: #333333;
    background-color: #f8f8f8;
    text-align: left;
    border-radius:4px;
}

input.noborder {
    border: 0;
}

input[disabled] {
    color: #888888;
}

option:disabled {
    color: #BBBBBB;
}

.modal img {
    max-width: 100%;
}
.modal table {
  width: 100%;
}
img {
  display: inline-block;
}
img.normal {
    border: white solid 1px;
}

img.alarm {
    border: red solid 1px;
}

hr {
    height: 1px;
    width: 100%;
    border: 0;
    color: var(--colorLink);
    background-color: var(--colorBackgroundButtons);
}

/*
 * Tabbed headings
 */
ul.tabList {
    float: left;
    list-style: none;
    padding: 0;
    margin: 0 0 -4px 0;
    white-space: nowrap;
    text-align: left;
}

ul.tabList li {
    float: left;
    border: 1px solid;
    color: #333333;
    border: #ccc solid 1px;
    border-bottom-width: 0;
    margin: 0 2px 0 0;
    background: #eee;
    text-align: center;
    padding: 5px 10px;
    border-top-left-radius: 3px;
    border-top-right-radius: 3px;
}

ul.tabList li a {
    text-decoration: none;
}

ul.tabList li:hover {
    background-color: #eeeeee;
}

ul.tabList li.active {
    background-color: #ffffff;
    border-bottom: #7f7fb2 dotted 1px;
}

ul.tabList li.active a {
  font-weight: bold;
}

#content {
  display: flex;
  flex-direction: column;
  width: 100%;
  /*"Pale moon" browser does not understand overflow: hidden auto;*/
  overflow-x: hidden;
  overflow-y: auto;
  margin: 0 auto 0 auto;
  line-height: 130%;
  text-align: center;
  clear: both;
}

#content.row{
  flex-direction: row;
}

#content::backdrop {
  background-color: white;
}

body.sticky #content {
  flex: 1 1 auto;
  /* whether or not #content is flex is a per-view thing
  display: flex;
  flex-flow: column;
  */
  height: 100%;
  /* There shouldn't be any, but if there is, at least we can scroll it
  overflow: hidden;
  */
}

#inner-content {
  overflow: auto;
  height: 100%;
  width: 100%;
}

/*
 * Major league table for multiple inputs or presentation
 */

#content table.major {
    margin: 4px auto;
    width: 100%;
    border-collapse: collapse;
}

#content table.major tr.highlight {
    background-color: #eeeeee;
}

#content table.major thead tr th {
    padding-top: 6px;
    padding-bottom: 6px;
    vertical-align: middle;
}

#content table.major tfoot td {
    padding-top: 6px;
    padding-bottom: 6px;
    vertical-align: middle;
}

#content table.major th, #content table.major td {
    border-bottom: 2px solid #f2f2f2;
    padding: 5px 10px;
    text-align: left;
}

#content table.major th {
    vertical-align: bottom;
}

#content table.major td {
    vertical-align: middle;
}

#content table.major th[scope=row] {
    padding: 4px 3px 3px;
    vertical-align: top;
}

#content table.major .colMark, #content table.major .colSelect {
    text-align: center;
}
/*
 * Lesser table for very simple forms
 */

#content table.minor {
    width: 280px;
    margin: 0 auto;
}

#content table.minor td {
    padding: 4px;
}

#content table.minor .colLeft {
    width: 50%;
    text-align: right;
}

#content table.minor .colRight {
    width: 50%;
    text-align: left;
}

#content table.minor input[type=submit]
{
    margin-top: 8px;
    padding: 0 2px;
    font-size: 120%;
}

#content table > tbody > tr:hover
{
    background-color: #EEE;
}

.overlay {
    font-size: 11px;
}

.overlay fieldset {
    background-color: #f8f8f8;
}

.validation-advice {
    padding: 4px;
    color: #dc143c;
}

/*
 * Behavior classes
 */

.ok, .infoText {
    color: #0fb9b1;
}
.alert, .warnText, .warning, .disabledText {
    color: #ffa801;
    padding: 0;
}


.alarm, .errorText, .error {
    color: #ff3f34;
}
/* Refers to the error box at the top of the web UI */
#error {
  width: 100%;
  padding: 5px;
  font-weight: bold;
  background-color: white;
  color: #ff3f34;
}

.timedErrorBox {
  color:white;
  background:#e74c3c;
  border-radius:5px;
  padding:5px;
  -moz-animation: inAndOut 5s ease-in forwards;
  -webkit-animation: inAndOut 5s ease-in forwards;
  animation: inAndOut 5s ease-in forwards;
}

/*
  the timed classed auto disappear after 5s
*/
.timedWarningBox {
  color:white;
  background:#e67e22;
  border-radius:5px;
  padding:5px;
  -moz-animation: inAndOut 5s ease-in forwards;
  -webkit-animation: inAndOut 5s ease-in forwards;
  animation: inAndOut 5s ease-in forwards;
}

.timedSuccessBox {
  color:white;
  background:#27ae60;
  border-radius:5px;
  padding:5px;
  -moz-animation: inAndOut 5s ease-in forwards; 
  -webkit-animation: inAndOut 5s ease-in forwards;
  animation: inAndOut 5s ease-in forwards;
}

@keyframes inAndOut {
  0% {opacity:0;}
  10% {opacity:1;}
  90% {opacity:1;}
  100% {opacity:0;}
}


.fakelink {
    color: #7f7fb2;
    cursor: pointer;
}

.fakelink:hover {
    color: #336699;
    text-decoration: none;
}

/*
 * Generic useful classes, especially with mootools
 */

.nowrap {
    white-space: nowrap;
}

div.clear,
br.clear {
    clear: both;
}

/*
.table-th-sort {
    font-style:italic;
}

td.table-td-sort {
    font-style:italic;
}
*/

th.table-th-sort {
    margin-right: 12px;
}

th.table-th-sort span.table-th-sort-span {
    float: right;
    width: 12px;
    height: 12px;
    background: url("../skins/classic/graphics/arrow-s-u.png") no-repeat 0 0;
}

th.table-th-sort-rev span.table-th-sort-span {
    float: right;
    width: 12px;
    height: 12px;
    background: url("../skins/classic/graphics/arrow-s-d.png") no-repeat 0 0;
}

.table-tr-odd {
    background-color: #f8f8f8;
}

/*
 * Primary layout styles
 */

#page {
    width: 100%;
}
body.sticky #page {
  flex: 1 1 auto;
  display: flex;
  flex-flow: column;
  overflow: hidden;
}

#header {
    width: 100%;
    background-color: #34495e;
    padding: 5px 0px;
    margin: 0 auto 4px auto;
    color: white;
    font-weight: 300;
}
#header:after {
  content: ".";
  display: block;
  height: 0;
    font-size: 0;
  clear: both;
  visibility: hidden;
}
@media screen and (max-width:767px) {
  #header {
    max-width: 100%;
  }
}


#header h2 {
    left: 0;
    font-weight: 300;
}

#header h2.floating {
    float: left;
    font-weight: 300;
}

#header h3 {
    font-weight: 300;
    font-size:0.8em;
}
#headerControl {
    text-align: center;
    margin: 0 auto;
}

#headerButtons {
    float: right;
}

#headerButtons a {
    margin-left: 8px;
}

#headerButtons input {
    margin-left: 4px;
}


#content p {
    margin-top: 4px;
}

#content p.textblock {
    text-align: justify;
    padding: 4px;
}

#content p.textblock br {
    margin-bottom: 8px;
}

/*
#contentDiv {
    margin: 0 auto 8px;
    line-height: 140%;
    text-align: center;
}
*/

#content > input[type=submit], #content > input[type=button], #content > button, #content > .btn-primary {
    margin-top: 8px;
}

#content table input[type=submit], #content table input[type=button], #content table button, #content table .btn-primary {
    margin-top: 1px;
    margin-right: 4px;
}

#contentButtons {
    margin-top: 8px;
    margin-bottom: 8px;
    float: right;
}

#contentButtons input {
    margin-left: 8px;
    margin-bottom: 4px;
}

#contentButtons a.btn, #contentButtons button.btn {
    margin-bottom: 0;
}

#contentForm {
    width: 100%;
    overflow: auto;
}

#footer {
    width: 96%;
    margin: 8px auto;
}

button,
input[type=button],
input[type=submit],
.btn-primary,
.btn-primary:link {
    background-color: var(--colorBackgroundButtons);
    color: #fff;
    border-color: var(--colorBorder);
    text-transform: uppercase;
    font-weight: 200;
    padding: 7px 10px 5px 10px;
    cursor: pointer;
    text-decoration: none;
    display: inline-block;
    text-align: center;
    margin-right: 3px;
}
/* Clear button for filter selects (general pages, not in extruder) */
.term-value-wrapper {
  position: relative;
  display: inline-block;
}

.btn-term-remove-all {
  position: absolute;
  top: 2px;
  right: 0px;
  z-index: 1011;
}

.btn-term-remove-all button {
  padding: 2px 4px;
  min-width: 24px;
  height: 24px;
  background-color: rgba(255, 255, 255, 0.9);
  border: 1px solid #ccc;
  border-radius: 3px;
  display: flex;
  align-items: center;
  justify-content: center;
  cursor: pointer;
}

.btn-term-remove-all button:hover {
  background-color: #f0f0f0;
  border-color: #999;
}

.btn-term-remove-all button .material-icons {
  font-size: 18px;
  color: #666;
}

.btn-term-remove-all button:hover .material-icons {
  color: #333;
}

.btn-normal,
.btn-normal:link {
    color: #fff;
    background-color: var(--colorBackgroundButtons);
    border: 1px solid var(--colorBorder);
}

button:hover,
input[type=button]:hover,
input[type=submit]:hover,
.btn-primary:hover {
    background-color: #34a2ee;    
}


/* PP - make it easy to identify disabled buttons */

button:disabled,
input[type=button]:disabled,
input[type=submit]:disabled,
  a.disabled,
  a.btn-primary.disabled,
.btn-primary.disabled, .btn-primary:disabled, .btn-secondary.disabled, .btn-secondary:disabled {
    background-color: #aaaaaa;    
    border-color: #bbbbbb;
}

#navbar-container {
  padding: 0;
  display: flex;
  justify-content: space-between;
  background-color: var(--backgroundMediumDark) !important;
}

#navbar-container > nav {
  width: 100%;
}

#reload, #panel {
  padding: 0;
}
#Version {
  padding-right: 8px;
}

.navbars {
  width: 100%;
}

.navbar {
  padding: 0 0 0 0px;
  margin-bottom: 0 !important;
  border-radius: 0;
  background-color: var(--backgroundMediumDark) !important;
  color: #0fbcf9;
  border:none;
}

.navbar-brand {
  margin-left: 15px;
  font-weight: normal;
  font-size: 30px;
}
.navbar-brand a {
  color:#ffa801;
}
@media (max-width: 600px){
  .navbar-brand {
    display: none;
  }
  .navbar-nav {
    flex-direction: row;
  }
  .navbar-nav .nav-link,
  #getHeaderFlipHTML a
  {
    padding-right: 0.5rem;
    padding-left: 0.5rem;
  }
}
#getHeaderFlipHTML a {
  display: block;
  padding: .5rem 1rem;
}

.navbar-btn {
  background-color:#95afc0;
  color:white;
  padding:4px;
  margin-right:8px;
  border:none;
}

.sidebar {
    position: absolute;
    top: 0;
    left: 0;
    z-index: 1000;
    display: block;
    padding: 0;
    overflow-x: hidden;
    overflow-y: auto;
    background-color: #f5f5f5;
    border-right: 1px solid #eeeeee;
}

.sidebar ul {
margin-right: 0;
margin-bottom: 20px;
margin-left: 0;
}

.nav-pills > li > a {
  border-radius: 0;
}

ul.nav.nav-pills.flex-column {
  background-color: #f5f5f5; 
}

#toolbar,
.filterTable {
  display:flex;
  flex-wrap: wrap;
}

.controlHeader .chosen-container,
#header .chosen-container,
#fbpanel .chosen-container,
#toolbar .chosen-container {
  text-align: left;
  min-width: 11em; /* Makes a nice uniform display */
}

#mfbpanel {
  width: calc(100% - 25px);
  float: left;
}

.chosen {
  position: absolute !important; left: -999em !important;
}

.chosen-container-active .chosen-choices {
    border-color:rgb(200,200,200);
}

.chosen-single,
.chosen-container,
.chosen-container-multi,
.chosen-results,
.chosen-search,
.chosen-drop,
.chosen-choices,
li.search-choice {
    font-size: inherit;
    border-radius:4px;
    border-color:rgb(200,200,200);
}

.chosen-container-single .chosen-single-with-deselect span {
  margin-right: 30px;
}

.chosen-container-single .chosen-single abbr {
  right: 18px;
}

.reduced-text { 
   font-size:0.9em;
}

.glyphicon.glyphicon-dot:before {
    content: "\25cf";
    font-size: 1.5em;
    padding-right:5px;
}

.gi-1p5x{font-size: 1.5em;}
.gi-2x{font-size: 2em;}
.gi-3x{font-size: 3em;}
.gi-4x{font-size: 4em;}
.gi-5x{font-size: 5em;}

.fa:hover{
  color: black;
}

.filterBar {
  margin-top:5px;
  margin-bottom:5px;
}
   
.form-check-inline {
  display: inline-block;
}

#dropdown_storage+div,
#dropdown_reminder+div,
#dropdown_bandwidth+div {
  background-color: var(--backgroundMediumDark);
}

#framesTable td:hover {
  cursor: pointer;
}

#thumb-overlay {
  position: fixed;
  top: 0;
  left: 0;
  width: 100vw;
  height: 100vh;
  display: flex;
  align-items: center;
  justify-content: center;
  z-index: 2000;
  pointer-events: none;
}

.thumb-overlay-img {
  border-radius: 4px;
  box-shadow: 0 4px 24px rgba(0, 0, 0, 0.5);
  background-size: cover;
  background-position: center;
  overflow: hidden;
}

.thumb-overlay-img img,
.thumb-overlay-img video {
  width: 100%;
  height: 100%;
  display: block;
  object-fit: cover;
}

.thumb-overlay-wrapper {
  display: flex;
  flex-direction: column;
  align-items: center;
}

.thumb-overlay-status {
  background: rgba(0, 0, 0, 0.85);
  color: #fff;
  padding: 10px 20px;
  margin-top: 16px;
  border-radius: 6px;
  font-family: monospace;
  font-size: 18px;
  min-height: 24px;
}

.live-indicator {
  display: flex;
  align-items: center;
  gap: 10px;
  font-weight: bold;
  color: #ff4444;
  text-transform: uppercase;
  letter-spacing: 2px;
  font-size: 20px;
}

.live-dot {
  width: 14px;
  height: 14px;
  background: #ff4444;
  border-radius: 50%;
  box-shadow: 0 0 8px rgba(255, 68, 68, 0.6);
  animation: live-pulse 1.2s ease-in-out infinite;
}

@keyframes live-pulse {
  0%, 100% {
    opacity: 1;
    transform: scale(1);
    box-shadow: 0 0 8px rgba(255, 68, 68, 0.6);
  }
  50% {
    opacity: 0.7;
    transform: scale(1.4);
    box-shadow: 0 0 16px rgba(255, 68, 68, 0.9);
  }
}

.time-indicator {
  display: flex;
  align-items: center;
  gap: 8px;
  color: #ccc;
}

.time-indicator .fa {
  color: #888;
}

.time-display {
  font-variant-numeric: tabular-nums;
}

a.flip {
  float: right;
  padding-right: 5px;
}

#content table.major .colDiskSpace {
  text-align: right;
}

#modalLogout .modal-dialog {
  height: 100%;
}
#modalLogout .modal-content {
  height: 90%;
}
#modalLogout .modal-body {
  overflow:auto;
}
#getAccountCircleHTML {
  padding: 4px 0 0 0;
}
#logoutButton {
  color: white;
  padding: 4px 10px 7px 10px;
}
#stateModalBtn {
  padding: 5px 10px 4px 10px;
}
#shutdownButton {
  padding: 4px 10px 4px 10px;
  color: white;
}
.imageFeed {
  border: 2px solid white; /* Because .alarm .imageFeed has a red border, so without this is jumps around */
  margin: 0 auto;
  text-align: center;
  /*
  vertical-align: middle;
  */
  position: relative;
}
.imageFeed img,
.imageFeed video,
.imageFeed svg {
  margin: 0;
  padding: 0;
  border-radius: 0;
  display: block;
  width: 100%;
  /*
  height: 100%;
  */
}
.imageFeed svg {
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  height: 100%;
  background: none;
}
.imageFeed svg.zones {
  z-index: 3;
}
.colThumbnail img {
  border-radius: 4px;
}
.monitor {
  position: relative;
}
.monitor:after {
  clear: both;
}

.monitor .monitorStatus:not(.hidden) {
  display: flex;
  flex-direction: row-reverse;
  justify-content: space-between;
  position: relative;
  background-color: #FFFFFF;
}

.monitor .monitorStatus.bottom {
  color: #dddddd;
}

.monitor.idle .imageFeed {
  border: 2px solid #ffffff;
}

.monitor.alarm .imageFeed {
  border: 2px solid #ff0000;
}

.monitor.alert .imageFeed {
  border: 2px solid #ffa500;
}
.monitor.alert {
  color: #ffa500;
  padding: 0;
  border-radius: 0;
  border: none;
}
.controlHeader, #fieldsTable {
  display: flex;
  flex-wrap: wrap;
  justify-content: center;
}
/* to match chosen text input height */
.controlHeader input[type="text"] {
  height: 29px;
  padding: 2px 5px 4px 5px;
  margin-top:0;
  width: 100%;
}
@media screen and (max-width:767px) {
  .controlHeader {
    display: flex;
    flex-wrap: wrap;
  }
}
#ui-datepicker-div {
  z-index:1001!important;
}

#fieldsTable ul.chosen-choices {
  max-height: 100px;
  overflow-y:auto;
}
.search .form-control {
  font-size: 100%;
}

/* input[type="search"]::-webkit-search-cancel-button {
  display: none;
} */

.tags-container {
  display: flex;
  border: 1px solid;
  border-color: #ccc;
  border-radius: 4px;
  min-height: 35px;
  margin: 0.25rem 1rem 0.25rem 1rem;
}

.tag {
  background-color: #F0F0F0;
  border-radius: 12px;
  padding: 4px 8px;
  margin: 4px;
  display: flex;
  align-items: center;
}

.tag-text {
  margin-right: 4px;
}

.tag-remove {
  cursor: pointer;
  color: red;
}

.tag-input {
  height: 30px;
  margin-left: 8px;
  border: none;
  width: 100%;
}

.tag-input:focus {
  outline: none;
}

.tag-dropdown {
  vertical-align: center;
  margin-right: 8px;
  display: inline-block;
  flex-grow: 2;
}

.tag-dropdown-content {
  display: none;
  position: absolute;
  background-color: #f9f9f9;
  min-width: 160px;
  box-shadow: 0px 8px 16px 0px rgba(0,0,0,0.2);
  z-index: 15;
  padding: 4px 0;
  overflow-y: auto;
  max-height: 800px;
}

.tag-dropdown-item {
  cursor: pointer;
  padding: 4px 8px;
  margin-bottom: 4px;
}

.tag-dropdown-item:hover {
  background-color: #dfdfdf;
}

.hlight{
  background:#dfdfdf;
}

.tag-input:focus + .tag-dropdown-content,
.tag-input + .tag-dropdown-content:active {
  display: block;
}

/* Change scrollbar style */
div::-webkit-scrollbar, nav::-webkit-scrollbar, .chosen-results::-webkit-scrollbar {
  width: 11px;
  height: 11px;
}

html, div, nav, .chosen-results {
  scrollbar-width: thin;
  scrollbar-color: var(--sliderBG) var(--scrollbarBG);
}

html::-webkit-scrollbar-track, div::-webkit-scrollbar-track, nav::-webkit-scrollbar-track, .chosen-results::-webkit-scrollbar-track {
  background: var(--scrollbarBG);
}

html::-webkit-scrollbar-thumb, div::-webkit-scrollbar-thumb, nav::-webkit-scrollbar-thumb, .chosen-results::-webkit-scrollbar-thumb {
  background-color: var(--sliderBG);
  border-radius: 6px;
  border: 3px solid var(--scrollbarBG);
}

.term {
  display: flex;
  flex-direction: column;
  margin-left: 15px;
}

@media screen and (max-width:500px) {
  .term {
    width: 100% !important;
    flex-direction: row;
    margin-right: 15px;
    margin-bottom: 5px;
  }
  .term-label-wrapper {
    text-wrap: nowrap;
    margin-top: 7px;
    margin-right: 5px;
  }
  .term-value-wrapper {
    width: 100%;
    flex-grow: 1;
  }
  .term-value-wrapper .chosen-container {
    width: 100% !important;
  }
  label {
    text-wrap: nowrap;
  }
  input.hasDatepicker {
    max-width: 100%;
    width: 100%;
  }
}

@media screen and (max-height:600px) {
  .sticky #mfbpanel {
    max-height: 40vh;
    overflow-y: auto;
    overflow-x: hidden;
  }

  .sticky #toolbar {
    max-height: 20vh;
    overflow-y: auto;
    overflow-x: hidden;
  }
}

/* +++ Control button block in the Stream image*/
.block-button-center {
  display: inline-block;
  position: relative;
  margin: 0 20%;
  top: 2px;
  z-index: 10;
}

button.btn.btn-zoom-out {
  padding: 0;
  background-color: rgba(150,150,150,0.5);
  position: absolute;
  left: 3px;
  top: 3px;
  z-index: 20;
  text-shadow:
    -1px -1px 0 #575757,
    1px -1px 0 #575757,
    -1px 1px 0 #575757,
    1px 1px 0 #575757;
}

button.btn.btn-zoom-in {
  padding: 0;
  background-color: rgba(150,150,150,0.5);
  position: absolute;
  right: 3px;
  top: 3px;
  z-index: 20;
  text-shadow:
    -1px -1px 0 #575757,
    1px -1px 0 #575757,
    -1px 1px 0 #575757,
    1px 1px 0 #575757;
}

button.btn.btn-view-watch,
button.btn.btn-edit-monitor,
button.btn.btn-fullscreen {
  padding: 0;
  background-color: rgba(150,150,150,0.5);
  z-index: 10;
  text-shadow:
    -1px -1px 0 #575757,
    1px -1px 0 #575757,
    -1px 1px 0 #575757,
    1px 1px 0 #575757;
}

button.btn.btn-zoom-out:focus,
button.btn.btn-zoom-in:focus,
button.btn.btn-view-watch:focus,
button.btn.btn-edit-monitor:focus {
  outline: 0;
  box-shadow: none;
}

button.btn.btn-zoom-out:hover,
button.btn.btn-zoom-in:hover,
button.btn.btn-edit-monitor:hover,
button.btn.btn-view-watch:hover {
  background-color: darkgrey;
}

.controlHeader .chosen-container {
  /* limit width of monitor filters so that it fits on 1920px */
  max-width: 300px;
}

/* --- Control button block in the Stream image*/

.fixed-t-r {
  position: fixed;
  top: 50px;
  right: 50px;
  min-width: 250px;
}

@media screen and (max-width:767px) {
  .fixed-t-r {
    position: fixed;
    top: 10px;
    right: 10px;
    min-width: 200px;
  }
}

/* +++ Style for <input> +++ */
@supports (-webkit-appearance: none) or (-moz-appearance: none) {
  input[type='checkbox'], input[type='radio'] {
    --active: #3498db;
    --active-inner: #eee;
    --focus: 2px rgba(39, 94, 254, 0.3);
    --border: #3498db;
    --border-hover: #3498db;
    --background: #eee;
    --disabled: #F6F8FF;
    --disabled-inner: #E1E6F9;
    -webkit-appearance: none;
    -moz-appearance: none;
    appearance: none;
    /*height: 21px;*/
    height: 18px;
    outline: none;
    display: inline-block;
    vertical-align: top;
    position: relative;
    margin: 0;
    cursor: pointer;
    border: 1px solid var(--bc, var(--border));
    background: var(--b, var(--background));
    transition: background 0.3s, border-color 0.3s, box-shadow 0.2s;
  }
  input[type='checkbox']:after, input[type='radio']:after {
    content: '';
    display: block;
    left: 0;
    top: 0;
    position: absolute;
    transition: transform var(--d-t, 0.3s) var(--d-t-e, ease), opacity var(--d-o, 0.2s);
  }
  input[type='checkbox']:checked, input[type='radio']:checked {
    --b: var(--active);
    --bc: var(--active);
    --d-o: 0.3s;
    --d-t: 0.6s;
    --d-t-e: cubic-bezier(0.2, 0.85, 0.32, 1.2);
  }
  input[type='checkbox']:disabled, input[type='radio']:disabled {
    --b: var(--disabled);
    cursor: not-allowed;
    opacity: 0.9;
  }
  input[type='checkbox']:disabled:checked, input[type='radio']:disabled:checked {
    --b: var(--disabled-inner);
    --bc: var(--border);
  }
  input[type='checkbox']:disabled + label, input[type='radio']:disabled + label {
    cursor: not-allowed;
  }
  input[type='checkbox']:hover:not(:checked):not(:disabled), input[type='radio']:hover:not(:checked):not(:disabled) {
    --bc: var(--border-hover);
  }
  input[type='checkbox']:focus, input[type='radio']:focus {
    box-shadow: 0 0 0 var(--focus);
  }
  input[type='checkbox']:not(.input-switch), input[type='radio']:not(.input-switch) {
    /*width: 21px;*/
    width: 18px;
  }
  input[type='checkbox']:not(.input-switch):after, input[type='radio']:not(.input-switch):after {
    opacity: var(--o, 0);
  }
  input[type='checkbox']:not(.input-switch):checked, input[type='radio']:not(.input-switch):checked {
    --o: 1;
  }
  input[type='checkbox'] + label, input[type='radio'] + label {
    font-size: 14px;
    line-height: 21px;
    display: inline-block;
    vertical-align: top;
    cursor: pointer;
    margin-left: 4px;
  }
  input[type='checkbox']:not(.input-switch) {
    border-radius: .2rem;
  }
  input[type='checkbox']:not(.input-switch):after {
    width: 5px;
    height: 9px;
    border: 2px solid var(--active-inner);
    border-top: 0;
    border-left: 0;
    /*left: 7px;*/
    left: 6px;
    /*top: 4px;*/
    top: 2px;
    transform: rotate(var(--r, 20deg));
  }
  input[type='checkbox']:not(.input-switch):checked {
    --r: 43deg;
  }
  input[type='checkbox'].input-switch {
    width: 38px;
    border-radius: 11px;
  }
  input[type='checkbox'].input-switch:after {
    left: 2px;
    top: 2px;
    border-radius: 50%;
    width: 15px;
    height: 15px;
    background: var(--ab, var(--border));
    transform: translateX(var(--x, 0));
  }
  input[type='checkbox'].input-switch:checked {
    --ab: var(--active-inner);
    --x: 17px;
  }
  input[type='checkbox'].input-switch:disabled:not(:checked):after {
    opacity: 0.6;
  }
  input[type='radio'] {
    border-radius: 50%;
  }
  input[type='radio']:after {
    position: static;
    /*width: 19px;*/
    width: 16px;
    /*height: 19px;*/
    height: 16px;
    border-radius: 50%;
    background: var(--active-inner);
    opacity: 0;
    transform: scale(var(--s, 0.7));
  }
  input[type='radio']:checked {
    --s: 0.5;
  }
}
/* --- Style for <input> --- */

/* +++ Volume сontrol +++ */
.audio-control-mute {
  position: relative;
  cursor: pointer;
  color: var(--colorBackgroundButtons);
  /*color: #DDD;
  float: right;
  right: 3px;
  margin: 5px;*/
}

.volumeSlider {
  position: relative;
  width: 80px;
  display: inline-block;
  margin: 0 10px 0 10px;
  /*float: right;
  right: 15px;
  margin-top: 10px;*/
}

/* 
noUi-target {
  border: 1px solid var(--scrollbarBG);
  box-shadow: inset 0 1px 1px #f0f0f0, 0 3px 6px -5px #bbb;
}
*/

[id^="volumeControls"]:not(.disabled) .noUi-base, [id^="volumeControls"]:not(.disabled) .noUi-touch-area {
  cursor: pointer;
}

.noUi-base .noUi-connect {
  background: var(--colorBackgroundButtons);
}

.noUi-mute .noUi-connect {
  background: #ccc;
}

.noUi-round {
  height: 10px;
}

.noUi-handle:before, .noUi-handle:after {
  height: 5px;
}

.noUi-handle:after {
  left: 16px;
}

/*
.noUi-round .noUi-connects {
  background: #c0392b;
}
*/

.noUi-round .noUi-handle {
  height: 18px;
  width: 18px;
  top: -5px;
  right: -9px; /* half the width */
  border-radius: 9px;
}

.disabled .audio-control-mute {
  color: var(--disabled);
  cursor: not-allowed !important;
}

[id^="volumeControls"].disabled {
  opacity: 0.4;
}
/* --- Volume сontrol --- */

.rightInFlexContainer {
  margin-left: auto; /* This will cause the element in the FLEX container to snap to the right side. All elements after this element will also snap to the right side. */
}

video-stream {
  position: relative;
}
.stream-info {
  /*
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  padding: 12px;
  */
  display: flex;
  justify-content: space-between;
  pointer-events: none;
}

.stream-info-status:empty, .stream-info-mode:empty {
  padding:0;
}

/*
.stream-info-status, .stream-info-mode {
  background-color: rgba(0,0,0,0.3);
  color: #dddddd;
  padding: 0.3em;
}
*/

/* Fix bootstrap-table */
.fixed-table-loading {
  overflow: hidden;
}

img[id^='liveStream'], video[id^='liveStream'], video-stream[id^='liveStream'] {
  margin: 0 auto;
  object-fit: fill;
}

video-stream[id^='liveStream'] video{
  max-height: inherit;
  max-width: inherit;
  object-fit: fill;
}

.imageFeed,
.nav,
.fixed-table-body,
*[id^='videoFeedStream']
 {
  border-radius: 4px;
}

/* +++ This block should always be located at the end! */
.hidden {
    display: none;
}

.hidden-shift {
  position: absolute !important; left: -999em !important;
}

.invisible {
    visibility: hidden;
}
/* --- This block should always be located at the end! */
