package ONVIF::Media::Types::VideoSourceConfiguration;
use strict;
use warnings;


__PACKAGE__->_set_element_form_qualified(1);

sub get_xmlns { 'http://www.onvif.org/ver10/schema' };

our $XML_ATTRIBUTE_CLASS;
undef $XML_ATTRIBUTE_CLASS;

sub __get_attr_class {
    return $XML_ATTRIBUTE_CLASS;
}


use base qw(ONVIF::Media::Types::ConfigurationEntity);
# Variety: sequence
use Class::Std::Fast::Storable constructor => 'none';
use base qw(SOAP::WSDL::XSD::Typelib::ComplexType);

Class::Std::initialize();

{ # BLOCK to scope variables

my %Name_of :ATTR(:get<Name>);
my %UseCount_of :ATTR(:get<UseCount>);
my %SourceToken_of :ATTR(:get<SourceToken>);
my %Bounds_of :ATTR(:get<Bounds>);
my %Extension_of :ATTR(:get<Extension>);

__PACKAGE__->_factory(
    [ qw(        Name
        UseCount
        SourceToken
        Bounds
        Extension

    ) ],
    {
        'Name' => \%Name_of,
        'UseCount' => \%UseCount_of,
        'SourceToken' => \%SourceToken_of,
        'Bounds' => \%Bounds_of,
        'Extension' => \%Extension_of,
    },
    {
        'Name' => 'ONVIF::Media::Types::Name',
        'UseCount' => 'SOAP::WSDL::XSD::Typelib::Builtin::int',
        'SourceToken' => 'ONVIF::Media::Types::ReferenceToken',
        'Bounds' => 'ONVIF::Media::Types::IntRectangle',
        'Extension' => 'ONVIF::Media::Types::VideoSourceConfigurationExtension',
    },
    {

        'Name' => 'Name',
        'UseCount' => 'UseCount',
        'SourceToken' => 'SourceToken',
        'Bounds' => 'Bounds',
        'Extension' => 'Extension',
    }
);

} # end BLOCK








1;


=pod

=head1 NAME

ONVIF::Media::Types::VideoSourceConfiguration

=head1 DESCRIPTION

Perl data type class for the XML Schema defined complexType
VideoSourceConfiguration from the namespace http://www.onvif.org/ver10/schema.






=head2 PROPERTIES

The following properties may be accessed using get_PROPERTY / set_PROPERTY
methods:

=over

=item * SourceToken


=item * Bounds


=item * Extension




=back


=head1 METHODS

=head2 new

Constructor. The following data structure may be passed to new():

 { # ONVIF::Media::Types::VideoSourceConfiguration
   SourceToken => $some_value, # ReferenceToken
   Bounds => ,
   Extension =>  { # ONVIF::Media::Types::VideoSourceConfigurationExtension
     Rotate =>  { # ONVIF::Media::Types::Rotate
       Mode => $some_value, # RotateMode
       Degree =>  $some_value, # int
       Extension =>  { # ONVIF::Media::Types::RotateExtension
       },
     },
     Extension =>  { # ONVIF::Media::Types::VideoSourceConfigurationExtension2
     },
   },
 },




=head1 AUTHOR

Generated by SOAP::WSDL

=cut

