@font-face {
  font-family: "Material Icons";
  font-style: normal;
  font-weight: 400;
  font-display: block;
  src: url("../../../../fonts/material-icons.woff2") format("woff2"), url("../../../../fonts/material-icons.woff") format("woff");
}
.material-icons {
  font-family: "Material Icons";
  font-weight: normal;
  font-style: normal;
  font-size: 24px;
  line-height: 1;
  letter-spacing: normal;
  text-transform: none;
  display: inline-block;
  white-space: nowrap;
  word-wrap: normal;
  direction: ltr;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  text-rendering: optimizeLegibility;
  font-feature-settings: "liga";
  vertical-align: middle;
  display: none;
}

.material-icons.md-8 { font-size: 8px; }
.material-icons.md-10 { font-size: 10px; }
.material-icons.md-12 { font-size: 12px; }
.material-icons.md-14 { font-size: 14px; }
.material-icons.md-16 { font-size: 16px; }
.material-icons.md-18 { font-size: 18px; }
.material-icons.md-20 { font-size: 20px; }
.material-icons.md-22 { font-size: 22px; }
.material-icons.md-24 { font-size: 24px; }
.material-icons.md-26 { font-size: 26px; }
.material-icons.md-28 { font-size: 28px; }
.material-icons.md-30 { font-size: 30px; }
.material-icons.md-32 { font-size: 32px; }
.material-icons.md-34 { font-size: 34px; }
.material-icons.md-36 { font-size: 36px; }
.material-icons.md-38 { font-size: 38px; }
.material-icons.md-40 { font-size: 40px; }

button .material-icons {
  font-size: 18px;
}
