#monitors .imageFeed img.idle {
  border: 2px solid #999999;
}

#monitors .imageFeed img.alarm {
  border: 2px solid #ff0000;
}

#monitors .imageFeed img.alert {
  border: 2px solid #ffa500;
}

.Active {
  stroke: #ff0000;
  fill: #ff0000;
}

.Inclusive {
  stroke: #FFA500;
  fill: #FFA500;
}

.Exclusive {
  stroke: #800080;
  fill: #800080;
}

.Preclusive {
  stroke: #0000FF;
  fill: #0000FF;
}
