body {
  background-color: #222222;
}

input[type="text"] {
  background-color: #222222;
  color: #dddddd;
}
input[type="text"]:focus,
input[type="password"]:focus {
  background-color: #0069d9;
  color: #dddddd;
}

input[type="password"] {
  background-color: #222222;
  color: #dddddd;
}

form {
  border: 1px solid #303030;
  background-color: #303030;
  box-shadow: 0 0 6px 0 rgba(0,0,0,0.08);
}

form .alarm {
border-bottom: 1px solid #303030;
}

.form-control::placeholder {
  color: #dddddd;
}
