
package WSDiscovery10::Attributes::Action;
use strict;
use warnings;

{ # BLOCK to scope variables

sub get_xmlns { 'http://schemas.xmlsoap.org/ws/2004/08/addressing' }

__PACKAGE__->__set_name('Action');
__PACKAGE__->__set_ref();
use base qw(
    SOAP::WSDL::XSD::Typelib::Attribute
    SOAP::WSDL::XSD::Typelib::Builtin::anyURI
);

}

1;


=pod

=head1 NAME

WSDiscovery10::Attributes::Action

=head1 DESCRIPTION

Perl data type class for the XML Schema defined attribute
Action from the namespace http://schemas.xmlsoap.org/ws/2004/08/addressing.







=head1 METHODS

=head2 new

 my $element = WSDiscovery10::Attributes::Action->new($data);

Constructor. The following data structure may be passed to new():

 { value => $value }

=head1 AUTHOR

Generated by SOAP::WSDL

=cut

