#inner-content {
  display: flex;
  justify-content: center;
}
#contentForm {
  height: 100%;
}

.Import {
  width: 50%;
  height: 100%;
}
.Results {
  height: 100%;
}

.AddMonitors {
  display: flex;
  position: relative;
  height: 100%;
}

th[data-field="camera.Manufacturer"],
th[data-field="camera.Model"],
th[data-field="camera.Monitor.Name"],
th[data-field="camera.monitor.Width"],
th[data-field="camera.monitor.Height"],
th[data-field="camera.monitor.Codec"],
th[data-field="Thumbnail"],
th[data-field="ip"] {
  text-align: left;
}

#toolbar {
  width: 100%;
  display: flex;
  align-items: center;
  justify-content: space-between;
}

#contentButtons {
  margin-left: auto;
  float:none;
  margin-top: 0;
}
